﻿using Microsoft.Win32;
using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Input;
using System.Windows.Media.Imaging;
using System.Windows.Threading;

namespace PreviewMediaElement
{
    /*
     *       _____ _____ _____                _       _
     *      |_   _/  __ \_   _|              (_)     | |
     *        | | | /  \/ | |  ___  ___   ___ _  __ _| |
     *        | | ||      | | / __|/ _ \ / __| |/ _` | |
     *       _| |_| \__/\ | |_\__ \ (_) | (__| | (_| | |
     *      |_____\_____/ |_(_)___/\___/ \___|_|\__,_|_|
     *                   ___
     *                  |  _|___ ___ ___
     *                  |  _|  _| -_| -_|  LICENSE
     *                  |_| |_| |___|___|
     *
     * IT NEWS  <>  PROGRAMMING  <>  HW & SW  <>  COMMUNITY
     *
     * This source code is a part of online courses at IT social
     * network WWW.ICT.SOCIAL
     *
     * Feel free to use it for whatever you want, modify it and share it but
     * don't forget to keep this link in your code.
     *
     * Visit http://www.ict.social/licenses for more information
     */

    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private Boolean playing = false;
        private Boolean sliding = false;
        private DispatcherTimer timer = new DispatcherTimer();
        private double speed = 1;
        public Boolean stopped = false;


        /// <summary>
        /// Class constructor
        /// </summary>
        /// <param name="index"></param>
        public MainWindow()
        {
            InitializeComponent();
            SetButtons(0);
            grControl.Visibility = Visibility.Hidden;

            timer.Interval = TimeSpan.FromSeconds(1);
            timer.Tick += timer_Tick;
            timer.Start();

            sdrVolume.Value = mePlayer.Volume;
        }


        #region Window procedures =======================================
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CloseWindow(object sender, RoutedEventArgs e)
        {
            wdwControls.Close();
        }


        /// <summary>
        /// Timer
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer_Tick(object sender, EventArgs e)
        {
            if ((mePlayer.Source != null) && (mePlayer.NaturalDuration.HasTimeSpan) && (!sliding))
            {
                sdrPointer.Minimum = 0;
                sdrPointer.Maximum = mePlayer.NaturalDuration.TimeSpan.TotalSeconds;
                sdrPointer.Value = mePlayer.Position.TotalSeconds;
            }
        }


        /// <summary>
        /// Shows the buttons bar (after mouse hover)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ShowButtons(object sender, MouseEventArgs e)
        {
            grControl.Visibility = Visibility.Visible;
        }


        /// <summary>
        /// Hides the buttons bar
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void HideButtons(object sender, MouseEventArgs e)
        {
            grControl.Visibility = Visibility.Hidden;
        }


        /// <summary>
        /// Setting access to the buttons
        /// </summary>
        /// <param name="status"></param>
        private void SetButtons(int status)
        {
            switch (status)
            {
                case 0:
                    sdrPointer.IsEnabled = false;
                    sdrPointer.Opacity = 0.5;
                    sdrVolume.IsEnabled = false;
                    sdrVolume.Opacity = 0.5;
                    btnPlay.IsEnabled = false;
                    btnPlay.Opacity = 0.5;
                    btnStop.IsEnabled = false;
                    btnStop.Opacity = 0.5;
                    btnMoveBack.IsEnabled = false;
                    btnMoveBack.Opacity = 0.5;
                    btnMoveForward.IsEnabled = false;
                    btnMoveForward.Opacity = 0.5;
                    btnSlow.IsEnabled = false;
                    btnSlow.Opacity = 0.5;
                    btnFast.IsEnabled = false;
                    btnFast.Opacity = 0.5;
                    btnNormal.IsEnabled = false;
                    btnNormal.Opacity = 0.5;
                    break;
                case 1:
                    sdrPointer.IsEnabled = false;
                    sdrPointer.Opacity = 0.5;
                    sdrVolume.IsEnabled = false;
                    sdrVolume.Opacity = 0.5;
                    btnPlay.IsEnabled = true;
                    btnPlay.Opacity = 1;
                    btnStop.IsEnabled = false;
                    btnStop.Opacity = 0.5;
                    btnMoveBack.IsEnabled = false;
                    btnMoveBack.Opacity = 0.5;
                    btnMoveForward.IsEnabled = false;
                    btnMoveForward.Opacity = 0.5;
                    btnSlow.IsEnabled = false;
                    btnSlow.Opacity = 0.5;
                    btnFast.IsEnabled = false;
                    btnFast.Opacity = 0.5;
                    btnNormal.IsEnabled = false;
                    btnNormal.Opacity = 0.5;
                    break;
                case 2:
                    sdrPointer.IsEnabled = true;
                    sdrPointer.Opacity = 1;
                    sdrVolume.IsEnabled = true;
                    sdrVolume.Opacity = 1;
                    btnPlay.IsEnabled = true;
                    btnPlay.Opacity = 1;
                    btnStop.IsEnabled = true;
                    btnStop.Opacity = 1;
                    btnMoveBack.IsEnabled = true;
                    btnMoveBack.Opacity = 1;
                    btnMoveForward.IsEnabled = true;
                    btnMoveForward.Opacity = 1;
                    btnSlow.IsEnabled = true;
                    btnSlow.Opacity = 1;
                    btnFast.IsEnabled = true;
                    btnFast.Opacity = 1;
                    btnNormal.IsEnabled = true;
                    btnNormal.Opacity = 1;
                    break;
                default:
                    break;
            }
        }


        /// <summary>
        /// Window closing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Close(object sender, System.ComponentModel.CancelEventArgs e)
        {
            StopPlaying();
        }


        /// <summary>
        /// Window closing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BtnClose(object sender, RoutedEventArgs e)
        {
            StopPlaying();
            wdwControls.Close();
        }


        /// <summary>
        /// Stop playing
        /// </summary>
        private void StopPlaying()
        {
            if (playing == false)
            {
                mePlayer.Position = TimeSpan.Zero;
            }
            else
            {
                mePlayer.Position = mePlayer.Position;
            }
            if (timer != null)
            {
                timer.Stop();
            }
            mePlayer.Stop();
            mePlayer.Close();
        }
        #endregion



        #region Slider procedures ====================================
        /// <summary>
        /// Slider start
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SdrU_Start(object sender, DragStartedEventArgs e)
        {
            sliding = true;
        }


        /// <summary>
        /// Slider end
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SdrU_End(object sender, DragCompletedEventArgs e)
        {
            sliding = false;
            mePlayer.Position = TimeSpan.FromSeconds(sdrPointer.Value);
        }


        /// <summary>
        /// Slider change
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SdrU_Change(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            tbkTime.Text = TimeSpan.FromSeconds(sdrPointer.Value).ToString(@"hh\:mm\:ss");
            tbkTotalTime.Text = TimeSpan.FromSeconds(sdrPointer.Maximum).ToString(@"hh\:mm\:ss");
        }


        /// <summary>
        /// Volume setting
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SdrVoice_Change(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            mePlayer.Volume = sdrVolume.Value;
            if (sdrVolume.Value == 0)
            {
                BitmapImage obr = new BitmapImage(
                            new Uri("pack://application:,,,/UkazkaMediaElement;component/Obr/bezZvuku.png"));
                imgObr.ImageSource = obr;
            }
            else
            {
                BitmapImage obr = new BitmapImage(
                            new Uri("pack://application:,,,/UkazkaMediaElement;component/Obr/zvuk.png"));
                imgObr.ImageSource = obr;
            }
        }
        #endregion



        #region Video procedures ======================================
        /// <summary>
        /// Video selection
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BtnOpen(object sender, RoutedEventArgs e)
        {
            OpenFileDialog openFileDialog = new OpenFileDialog();
            openFileDialog.Filter = "Video files (*.mpg;*.mpeg;*.avi;*.mp4)|*.mpg;*.mpeg;*.avi;*.mp4";
            if (openFileDialog.ShowDialog() == true)
            {
                CancelPlaying();
                mePlayer.Source = new Uri(openFileDialog.FileName);
                lblFile.Content = openFileDialog.FileName;
                SetButtons(1);
            }
        }


        /// <summary>
        /// Play or pause the video
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BtnPlay(object sender, RoutedEventArgs e)
        {
            timer.Start();
            if (playing == false)
            {
                Play();
                SetButtons(3);
                stopped = false;
            }
            else
            {
                Pause();
                SetButtons(1);
            }
        }


        /// <summary>
        /// Stop the video
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BtnStop(object sender, RoutedEventArgs e)
        {
            if (timer != null)
            {
                timer.Stop();
            }
            mePlayer.Stop();
            sdrPointer.Value = 0;
            playing = false;
            btnPlay.SetResourceReference(Button.StyleProperty, "Play");
            btnPlay.ToolTip = "Play video";
            SetButtons(1);
            stopped = true;
        }


        /// <summary>
        /// Play the video
        /// </summary>
        private void Play()
        {
            mePlayer.Play();
            playing = true;
            btnPlay.SetResourceReference(Button.StyleProperty, "Pause");
            btnPlay.ToolTip = "Pause video";
            SetButtons(2);
        }


        /// <summary>
        /// Pause the video
        /// </summary>
        private void Pause()
        {
            mePlayer.Pause();
            playing = false;
            btnPlay.SetResourceReference(Button.StyleProperty, "Play");
            btnPlay.ToolTip = "Play video";
            SetButtons(1);
        }


        /// <summary>
        /// Rewind video back
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BtnBack(object sender, RoutedEventArgs e)
        {
            mePlayer.Position -= TimeSpan.FromSeconds(10);
        }


        /// <summary>
        /// Moves the video forward
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BtnForward(object sender, RoutedEventArgs e)
        {
            mePlayer.Position += TimeSpan.FromSeconds(10);

        }


        /// <summary>
        /// Slow play
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BtnSlow(object sender, RoutedEventArgs e)
        {
            int currentSpeed = 0;
            if (speed > 1)
            {
                speed -= speed / 2;
                mePlayer.SpeedRatio = speed;
            }
            else
            {
                if (speed == 1)
                {
                    speed -= 0.2;
                    mePlayer.SpeedRatio = speed;
                }
                else
                {
                    currentSpeed = (int)(speed * 10);
                    if (currentSpeed > 2)
                    {
                        speed -= 0.2;
                        mePlayer.SpeedRatio = speed;
                    }
                }
            }
            tbkSpeed.Text = speed.ToString();
        }


        /// <summary>
        /// Fast play
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BtnFast(object sender, RoutedEventArgs e)
        {
            if (speed >= 1 && speed < 32)
            {
                speed += speed;
                mePlayer.SpeedRatio = speed;
            }
            else
            {
                if (speed < 1)
                {
                    speed += 0.2;
                    mePlayer.SpeedRatio = speed;
                }
            }
            tbkSpeed.Text = speed.ToString();
        }


        /// <summary>
        /// Normal speed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BtnNormal(object sender, RoutedEventArgs e)
        {
            speed = 1;
            mePlayer.SpeedRatio = speed;
            tbkSpeed.Text = speed.ToString();
        }


        /// <summary>
        /// Procedure for video end
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SetEnd(object sender, RoutedEventArgs e)
        {
            mePlayer.Stop();
            sdrPointer.Value = 0;
            sdrPointer.Refresh();
            playing = false;
            sliding = false;
            stopped = false;
            btnPlay.SetResourceReference(Button.StyleProperty, "Play");
            btnPlay.ToolTip = "Play video";
            SetButtons(1);
        }


        /// <summary>
        /// Cancel video playing
        /// </summary>
        private void CancelPlaying()
        {
            mePlayer.Stop();
            mePlayer.Close();
            sdrPointer.Value = 0;
            sdrPointer.Refresh();
            tbkTime.Text = "00:00:00";
            tbkTotalTime.Text = "00:00:00";
            tbkSpeed.Text = "1";
            speed = 1;
            mePlayer.SpeedRatio = speed;
            playing = false;
            sliding = false;
            stopped = false;
            btnPlay.SetResourceReference(Button.StyleProperty, "Play");
            btnPlay.ToolTip = "Play video";
            SetButtons(0);
        }
        #endregion
    }
}
