﻿using System;
using System.IO;
using System.Windows;
using System.Windows.Controls;

namespace PreviewTreeView
{
    /*
     *       _____ _____ _____                _       _
     *      |_   _/  __ \_   _|              (_)     | |
     *        | | | /  \/ | |  ___  ___   ___ _  __ _| |
     *        | | ||      | | / __|/ _ \ / __| |/ _` | |
     *       _| |_| \__/\ | |_\__ \ (_) | (__| | (_| | |
     *      |_____\_____/ |_(_)___/\___/ \___|_|\__,_|_|
     *                   ___
     *                  |  _|___ ___ ___
     *                  |  _|  _| -_| -_|  LICENSE
     *                  |_| |_| |___|___|
     *
     * IT NEWS  <>  PROGRAMMING  <>  HW & SW  <>  COMMUNITY
     *
     * This source code is a part of online courses at IT social
     * network WWW.ICT.SOCIAL
     *
     * Feel free to use it for whatever you want, modify it and share it but
     * don't forget to keep this link in your code.
     *
     * Visit http://www.ict.social/licenses for more information
     */

    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private string activeDirectory = "";
        private string selected = null;
        private TreeRow selectedTree = null;

        
        /// <summary>
        /// Class constructor
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();
            LoadData();
        }


        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CloseWindow(object sender, RoutedEventArgs e)
        {
            wdwControls.Close();
        }


        /// <summary>
        /// Displaying selection from simple tree
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void DisplaySelection1(object sender, RoutedPropertyChangedEventArgs<object> e)
        {
            var tv = sender as TreeView;
            if (tv != null)
            {
                if (tv.Parent != null)
                {
                    TreeViewItem tvi = new TreeViewItem();
                    tvi = tv.SelectedItem as TreeViewItem;
                    if (String.IsNullOrEmpty(tvi.Uid))
                    {
                        lblSelected1.Content = tvi.Header;
                    }
                    else
                    {
                        lblSelected1.Content = "";
                    }
                }
            }
        }


        #region Tree created from code =====================================
        /// <summary>
        /// Adding discs list into tree
        /// </summary>
        private void LoadData()
        {
            tvwDirectories.Items.Clear();
            DriveInfo[] disks = DriveInfo.GetDrives();
            foreach (DriveInfo disc in disks)
            {
                TreeRow psDisc = new TreeRow();
                psDisc.Header = disc.Name;
                psDisc.Path = disc.Name;
                psDisc.Type = disc.GetType().Name;
                psDisc.Items.Add("Adding...");
                psDisc.Expanded += new RoutedEventHandler(Extract);
                psDisc.SetResourceReference(TreeViewItem.StyleProperty, "Header");
                tvwDirectories.Items.Add(psDisc);
            }
        }


        /// <summary>
        /// Extracts tree row
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Extract(object sender, RoutedEventArgs e)
        {
            TreeRow tvi = (TreeRow)sender;
            CreateTreeRow(tvi);
            e.Handled = true;
        }


        /// <summary>
        /// Creates tree row
        /// </summary>
        /// <param name="row"></param>
        private void CreateTreeRow(TreeRow row)
        {
            DirectoryInfo di = new DirectoryInfo(row.Path);
            row.Items.Clear();

            foreach (DirectoryInfo dir in di.GetDirectories())
            {
                var hidden = (dir.Attributes & FileAttributes.Hidden) == FileAttributes.Hidden;
                var system = (dir.Attributes & FileAttributes.System) == FileAttributes.System;
                if (!hidden && !system)
                {
                    TreeRow tvi = new TreeRow();
                    tvi.Header = dir.Name;
                    tvi.Path = dir.FullName;
                    tvi.Type = dir.GetType().Name;
                    tvi.Expanded += new RoutedEventHandler(Extract);
                    tvi.SetResourceReference(TreeViewItem.StyleProperty, "Item");
                    try
                    {
                        foreach (DirectoryInfo directory in dir.GetDirectories())
                        {
                            TreeRow ps = new TreeRow();
                            ps.Expanded += new RoutedEventHandler(Extract);
                            ps.Header = directory.Name;
                            ps.Path = dir.FullName;
                            ps.Type = directory.GetType().Name;
                            tvi.Items.Add(ps);
                        }
                        row.Items.Add(tvi);
                    }
                    catch (Exception)
                    {

                    }
                }
            }
        }


        /// <summary>
        /// Read directories
        /// </summary>
        /// <param name="psV"></param>
        public void ReadDirectories(TreeRow psV)
        {
            if (!String.IsNullOrEmpty(psV.Path))
            {
                CreateTreeRow(psV);
                psV.IsExpanded = true;

                foreach (TreeRow ps2 in psV.Items)
                {
                    if (activeDirectory.IndexOf(ps2.Path) > -1)
                    {
                        ps2.IsSelected = true;
                        lblSelected2.Content = ps2.Path;
                        ReadDirectories(ps2);
                    }
                }
            }
        }


        /// <summary>
        /// Select directory
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SelectDirectory(object sender, RoutedPropertyChangedEventArgs<object> e)
        {
            if (e.NewValue != null)
            {
                TreeViewItem sel = tvwDirectories.SelectedItem as TreeViewItem;
                selectedTree = tvwDirectories.SelectedItem as TreeRow;

                TreeRow ps = (TreeRow)e.NewValue;
                lblSelected2.Content = ps.Path;
                selected = ps.Path;
            }
            e.Handled = true;
        }
        #endregion
    }


    /// <summary>
    /// Tree record structure
    /// </summary>
    public class TreeRow : TreeViewItem
    {
        public string Path { get; set; }
        public string Type { get; set; }
    }
}
